"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sanitizeInteractionNodes = exports.sanitizeInteractionType = exports.extractPageviewKey = exports.sanitizePageviewProperties = exports.sanitizeTimestamp = exports.sanitizeSourceInfo = exports.isNonEmptyString = exports.sanitizeProperties = void 0;
var types_1 = require("./types");
function sanitizeProperties(properties, logger, logSource) {
    var e_1, _a;
    var _b;
    if (!properties) {
        return null;
    }
    if (typeof properties !== 'object') {
        logger.debug('Ignoring properties because they are not an object', logSource);
        return null;
    }
    var output = {};
    try {
        for (var _c = __values(Object.keys(properties)), _d = _c.next(); !_d.done; _d = _c.next()) {
            var key = _d.value;
            var value = properties[key];
            if (!(0, types_1.isPropertyValue)(value)) {
                logger.debug("Ignoring property ".concat(key, " because the value ").concat(((_b = value === null || value === void 0 ? void 0 : value.toString) === null || _b === void 0 ? void 0 : _b.call(value)) || value, " is not a valid property type"), logSource);
                continue;
            }
            output[key] = value.toString();
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return output;
}
exports.sanitizeProperties = sanitizeProperties;
function isNonEmptyString(value) {
    return value && typeof value === 'string';
}
exports.isNonEmptyString = isNonEmptyString;
function sanitizeSourceInfo(source, logger, logSource) {
    if (!source) {
        return null;
    }
    if (!isNonEmptyString(source.name)) {
        logger.debug('Source will be ignored because name was omitted.', logSource);
        return null;
    }
    if (!isNonEmptyString(source.version)) {
        logger.debug('Source will be ignored because version was omitted.', logSource);
        return null;
    }
    if (!isNonEmptyString(source.platform)) {
        logger.debug('Source will be ignored because platform was omitted.', logSource);
        return null;
    }
    var copy = {
        name: source.name,
        version: source.version,
        platform: source.platform,
    };
    var properties = sanitizeProperties(source.properties, logger, logSource);
    if (properties) {
        copy.properties = properties;
    }
    return copy;
}
exports.sanitizeSourceInfo = sanitizeSourceInfo;
function sanitizeTimestamp(value, logger, logSource) {
    if (!value) {
        return null;
    }
    if (typeof value.getTime !== 'function') {
        logger.debug('Timestamp will be ignored because it is not a date.', logSource);
        return null;
    }
    return value.getTime();
}
exports.sanitizeTimestamp = sanitizeTimestamp;
function sanitizePageviewProperties(properties, logger, logSource) {
    if (!properties) {
        return null;
    }
    if (typeof properties !== 'object') {
        logger.debug('Ignoring pageview properties because they are not an object', logSource);
        return null;
    }
    var output = {};
    var componentOrClassName = properties.componentOrClassName;
    var title = properties.title;
    var url = properties.url;
    var sourceProperties = sanitizeProperties(properties.sourceProperties, logger, logSource);
    if (isNonEmptyString(componentOrClassName)) {
        output.componentOrClassName = componentOrClassName;
    }
    if (isNonEmptyString(title)) {
        output.title = title;
    }
    if (isNonEmptyString(url)) {
        output.url = url;
    }
    if (sourceProperties) {
        output.sourceProperties = sourceProperties;
    }
    return output;
}
exports.sanitizePageviewProperties = sanitizePageviewProperties;
function extractPageviewKey(value, logger, logSource) {
    if (!value) {
        return null;
    }
    if (value === 'none') {
        return 'none';
    }
    var pageviewKey = value.pageviewKey;
    if (!isNonEmptyString(pageviewKey)) {
        logger.debug('Pageview did not have a valid key and will be ignored.', logSource);
        return null;
    }
    return pageviewKey;
}
exports.extractPageviewKey = extractPageviewKey;
function sanitizeInteractionType(interaction, logger, logSource) {
    if (!interaction) {
        return null;
    }
    if (isNonEmptyString(interaction)) {
        return interaction;
    }
    var builtin = interaction.builtin;
    if (typeof interaction.builtin === 'number') {
        return { builtin: builtin };
    }
    var custom = interaction.custom;
    if (isNonEmptyString(custom)) {
        return { custom: custom };
    }
    logger.debug('Interaction type was of an unknown format.', logSource);
    return null;
}
exports.sanitizeInteractionType = sanitizeInteractionType;
function sanitizeInteractionNodes(nodes, logger, logSource) {
    if (!nodes || !Array.isArray(nodes) || nodes.length === 0) {
        return null;
    }
    var transformed = nodes.flatMap(function (node) {
        var sanitizedNode = sanitizeInteractionNode(node, logger, logSource);
        return sanitizedNode ? [sanitizedNode] : [];
    });
    if (transformed.length === 0) {
        return null;
    }
    return transformed;
}
exports.sanitizeInteractionNodes = sanitizeInteractionNodes;
function sanitizeInteractionNode(node, logger, logSource) {
    if (!node) {
        return null;
    }
    var nodeName = node.nodeName;
    if (!isNonEmptyString(nodeName)) {
        logger.debug('Interaction node omitted required field nodeName and will be ignored.', logSource);
        return null;
    }
    var result = { nodeName: nodeName };
    var propertyNames = [
        'nodeText',
        'nodeHtmlClass',
        'nodeId',
        'href',
        'accessibilityLabel',
        'referencingPropertyName',
    ];
    propertyNames.forEach(function (key) {
        var value = node[key];
        if (isNonEmptyString(value)) {
            result[key] = value;
        }
    });
    var attributes = sanitizeProperties(node.attributes, logger, logSource);
    if (attributes) {
        result.attributes = attributes;
    }
    return result;
}
